@@ -1,21 +1,25 @@
 package ElasticSearch::Error;
 {
-  $ElasticSearch::Error::VERSION = '0.47';
+  $ElasticSearch::Error::VERSION = '0.48';
 }
 
-@ElasticSearch::Error::Internal::ISA    = __PACKAGE__;
-@ElasticSearch::Error::Param::ISA       = __PACKAGE__;
-@ElasticSearch::Error::NoServers::ISA   = __PACKAGE__;
-@ElasticSearch::Error::Request::ISA     = __PACKAGE__;
-@ElasticSearch::Error::Timeout::ISA     = __PACKAGE__;
-@ElasticSearch::Error::Connection::ISA  = __PACKAGE__;
-@ElasticSearch::Error::JSON::ISA        = __PACKAGE__;
-@ElasticSearch::Error::QueryParser::ISA = __PACKAGE__;
+@ElasticSearch::Error::Internal::ISA       = __PACKAGE__;
+@ElasticSearch::Error::Param::ISA          = __PACKAGE__;
+@ElasticSearch::Error::NoServers::ISA      = __PACKAGE__;
+@ElasticSearch::Error::ClusterBlocked::ISA = __PACKAGE__;
+@ElasticSearch::Error::Request::ISA        = __PACKAGE__;
+@ElasticSearch::Error::Timeout::ISA        = __PACKAGE__;
+@ElasticSearch::Error::Connection::ISA     = __PACKAGE__;
+@ElasticSearch::Error::JSON::ISA           = __PACKAGE__;
+@ElasticSearch::Error::QueryParser::ISA    = __PACKAGE__;
 @ElasticSearch::Error::Conflict::ISA
     = ( 'ElasticSearch::Error::Request', __PACKAGE__ );
 @ElasticSearch::Error::Missing::ISA
     = ( 'ElasticSearch::Error::Request', __PACKAGE__ );
 
+@ElasticSearch::Error::NotReady::ISA
+    = ( 'ElasticSearch::Error::Connection', __PACKAGE__ );
+
 use strict;
 use warnings FATAL => 'all', NONFATAL => 'redefine';
 
@@ -82,7 +86,19 @@ The request timed out
 
 =item * ElasticSearch::Error::Connection
 
-There was an error connecting to the current server
+There was an error connecting to the current node. The request will be
+retried on another node.
+
+=item * ElasticSearch::Error::NotReady
+
+The current node is not yet able to serve requests. The request will be
+retried on another node. C<ElasticSearch::Error::NotReady> inherits from
+C<ElasticSearch::Error::Connection>.
+
+=item * ElasticSearch::Error::ClusterBlocked
+
+The cluster was unable to process the request because it is currently blocking,
+eg the requested index is closed.
 
 =item * ElasticSearch::Error::Request
 
