@@ -1,6 +1,6 @@
 package ElasticSearch::Transport::HTTP;
 {
-  $ElasticSearch::Transport::HTTP::VERSION = '0.47';
+  $ElasticSearch::Transport::HTTP::VERSION = '0.48';
 }
 
 use strict;
@@ -42,12 +42,12 @@ sub send_request {
 
     my $msg  = $server_response->message;
     my $code = $server_response->code;
-    my $type
-        = $code eq '409'          ? 'Conflict'
-        : $code eq '404'          ? 'Missing'
-        : $msg  eq 'read timeout' ? 'Timeout'
+    my $type = $self->code_to_error($code)
+        || (
+          $msg eq 'read timeout' ? 'Timeout'
         : $msg =~ /Can't connect|Server closed connection/ ? 'Connection'
-        :                                                    'Request';
+        : 'Request'
+        );
     my $error_params = {
         server      => $server,
         status_code => $code,
