@@ -12,7 +12,9 @@ ok $r = $es->cluster_health( as_json => 1 ), 'As JSON';
 isa_ok $json->decode($r), 'HASH', ' - is JSON';
 
 ok $r = $es->bulk_index(
-    [ { index => 'es_test_1', type => 'type_1', data => { text => 'foo' } } ],
+    docs => [
+        { index => 'es_test_1', type => 'type_1', data => { text => 'foo' } }
+    ],
     as_json => 1
     ),
     ' - bulk as JSON';
